(in-package :asn.1)

(defvar *reserved-words-table* (make-hash-table))

(defun fill-reserved-words ()
  (dolist (i *reserved-words*)
    (setf (gethash i *reserved-words-table*) i)))

(eval-when (:load-toplevel :execute)
  (fill-reserved-words))

(defun asn.1-lexer (stream)
  (let ((*readtable* *asn.1-readtable*))
    (let ((token (read stream nil nil nil)))
      (if token
        (values (detect-token token) token)
        (values nil nil)))))

(defgeneric detect-token (token))

(defmethod detect-token ((token number))
  :number)

(defmethod detect-token ((token symbol))
  (gethash token *reserved-words-table* :id))

(defmethod detect-token ((token string))
  :string)

(defgeneric parse (source))

(defmethod parse ((source string))
  (with-input-from-string (s source)
    (parse s)))

(defmethod parse ((source pathname))
  (with-open-file (s source :direction :input)
    (parse s)))

(defmethod parse ((source stream))
  (asn.1-parser #'(lambda () (asn.1-lexer source))))

(defmethod parse ((source t))
  (error "Unknown Parser Source"))
