;;;; ASN.1 Syntax (BNF -> Lisp)

(in-package :asn.1)

(defparameter *reserved-words*
  '(ABSENT ENCODED INTEGER RELATIVE-OID
    ABSTRACT-SYNTAX END INTERSECTION SEQUENCE 
    ALL ENUMERATED |ISO646String| SET
    APPLICATION EXCEPT MAX SIZE
    AUTOMATIC EXPLICIT MIN STRING
    BEGIN EXPORTS MINUS-INFINITY SYNTAX 
    BIT EXTENSIBILITY NULL |T61String|
    |BMPString| EXTERNAL |NumericString| TAGS
    BOOLEAN FALSE OBJECT |TeletexString|
    BY FROM |ObjectDescriptor| TRUE
    CHARACTER |GeneralizedTime| OCTET TYPE-IDENTIFIER
    CHOICE |GeneralString| OF UNION
    CLASS |GraphicString| OPTIONAL UNIQUE
    COMPONENT |IA5String| PATTERN UNIVERSAL
    COMPONENTS IDENTIFIER PDV |UniversalString|
    CONSTRAINED IMPLICIT PLUS-INFINITY |UTCTime|
    CONTAINING IMPLIED PRESENT |UTF8String|
    DEFAULT IMPORTS |PrintableString| |VideotexString|
    DEFINITIONS INCLUDES PRIVATE |VisibleString|
    EMBEDDED INSTANCE REAL WITH
    ;; Additional keywords
    |::=| { } |,| |(| |)| |;| < >
    MACRO TYPE VALUE NOTATION
    MODULE-IDENTITY OBJECT-TYPE NOTIFICATION-TYPE
    TEXTUAL-CONVENTION MODULE-COMPLIANCE OBJECT-GROUP
    NOTIFICATION-GROUP OBJECT-IDENTITY))

(defparser asn.1-parser
  ((root module-definition) $1)
  ;;((root assignment) $1)
  ((module-definition :id
                      DEFINITIONS
                      tag-default
                      extension-default
                      |::=|
                      BEGIN
                      module-body
                      END)
   `(defmodule ,$1 (,$3 ,$4) ,@$7))
  ((tag-default))
  ((extension-default))
  ((module-body exports imports assignment-list)
   `(,$1 ,$2 ,$3))
  ((module-body))
  ((exports EXPORTS ALL |;|) `(export :all))
  ;; ((exports EXPORTS symbol* |;|) `(export ,@$2))
  ((exports))
  ((imports IMPORTS symbols-from-modules |;|) `(import ,@$2))
  ((imports))
  ((symbols-from-modules symbols-from-modules symbols-from-module) `(,@$1 ,$2))
  ((symbols-from-modules symbols-from-module) `(,$1))
  ((symbols-from-module symbol+ FROM :id) `(,$1 :from ,$3))
  ((assignment-list assignment-list assignment) `(,@$1 ,$2))
  ((assignment-list assignment) `(,$1))
  ;;((assignment type-assignment) $1)
  ;;((assignment value-assignment) $1)
  ((assignment macro-definition) $1)
  ;;((type-assignment))
  ;;((value-assignment))
  ;; ASN.1 Macro
  ((macro-definition macro-name MACRO |::=|
                                BEGIN
                                general-list
                                END) `(macro ,$1))
  ;; Symbol+
  ((symbol+ symbol+ |,| symbol) `(,@$1 ,$3))
  ((symbol+ symbol) `(,$1))
  ((symbol macro-name) $1)
  ((symbol :id) $1)
  ((macro-name MODULE-IDENTITY) $1)
  ((macro-name OBJECT-TYPE) $1)
  ((macro-name NOTIFICATION-TYPE) $1)
  ((macro-name TEXTUAL-CONVENTION) $1)
  ((macro-name MODULE-COMPLIANCE) $1)
  ((macro-name OBJECT-GROUP) $1)
  ((macro-name NOTIFICATION-GROUP) $1)
  ((macro-name OBJECT-IDENTITY) $1)
  ;;; Last Rules
  ((general-list general) `(,$1))
  ((general-list general-list |,| general) `(,@$1 ,$2))
  ((general-list general-list general) `(,@$1 ,$2))
  ((general :number) $1)
  ((general :id) $1)
  ((general :string) $1)
  ((general |::=|) $1)
  ((general |(|) $1)
  ((general |)|) $1)
  ((general {) $1)
  ((general }) $1)
  ((general <) $1)
  ((general >) $1)
  ((general TYPE) $1)
  ((general VALUE) $1)
  ((general NOTATION) $1)
  ((general SEQUENCE) $1)
  ((general OBJECT) $1)
  ((general IDENTIFIER) $1)
  ((general |IA5String|) $1))
